@@ -161,11 +161,13 @@ class TestLinker(tests.TestCase):
 
 		self.assertEqual(linker.link('+dus'), './bar/dus.html')
 		self.assertEqual(linker.link('dus'), './dus.html')
+		self.assertEqual(linker.link('#heading'), '#heading')
+		self.assertEqual(linker.link('dus#heading'), './dus.html#heading')
 		self.assertEqual(linker.link('./dus.pdf'), './bar/dus.pdf')
 		self.assertEqual(linker.link('../dus.pdf'), './dus.pdf')
 		self.assertEqual(linker.link('../../dus.pdf'), '../dus.pdf')
 
-		extpath = 'C:\\dus.pdf' if os.name == 'nt' else '/duf.pdf'
+		extpath = 'C:\\dus.pdf' if os.name == 'nt' else '/dus.pdf'
 		self.assertEqual(linker.link(extpath), FilePath(extpath).uri)
 
 		# TODO:
@@ -350,6 +352,29 @@ Some content here
 		self.assertTrue(proxy.content.startswith('<h1>'))
 
 
+class TestPageProxyWithEmptyPage(tests.TestCase):
+
+	# Note: specifically test there are no exceptions here ...
+
+	CONTENT = {
+		'test': ''
+	}
+
+	def runTest(self):
+		from zim.formats import StubLinker
+		notebook = self.setUpNotebook(content=self.CONTENT)
+		page = notebook.get_page(Path('test'))
+		dumper = get_format('html').Dumper()
+		linker = StubLinker()
+		proxy = PageProxy(notebook, page, dumper, linker)
+
+		self.assertEqual(proxy.title, 'test')
+		self.assertEqual(proxy.heading, '')
+		self.assertIsInstance(proxy.body, str)
+		self.assertIsInstance(proxy.content, str)
+
+
+
 class TestPageSelections(tests.TestCase):
 
 	def _test_iface(self, selection):
@@ -710,9 +735,9 @@ class TestExportDialog(tests.TestCase):
 
 		with tests.LoggingFilter(logger='zim', message='Test'):
 			with log_context:
-				mylogger.warn('Test export warning')
+				mylogger.warning('Test export warning')
 				mylogger.debug('Test export debug')
-				foologger.warn('Test foo')
+				foologger.warning('Test foo')
 
 		file = log_context.file
 		self.assertTrue(file.exists())
