@@ -1,9 +1,5 @@
 
-# Copyright 2012-2020 Jaap Karssenberg <jaap.karssenberg@gmail.com>
-
-'''Test cases for the base zim module.'''
-
-
+# Copyright 2012-2022 Jaap Karssenberg <jaap.karssenberg@gmail.com>
 
 
 import tests
@@ -23,6 +19,9 @@ import zim.main
 import zim.main.ipc
 
 
+from zim.notebook.info import NotebookInfo
+
+
 class capture_stdout:
 
 		def __enter__(self):
@@ -77,26 +76,90 @@ class TestHelp(tests.TestCase):
 
 class TestNotebookCommand(tests.TestCase):
 
+	cmd_class = NotebookCommand
 
-	def runTest(self):
-		cmd = NotebookCommand('gui')
+	def get_cmd(self, *args):
+		cmd = self.cmd_class('gui')
 		cmd.arguments = ('NOTEBOOK', '[PAGE]')
-		cmd.parse_options('./Notes')
+		cmd.parse_options(*args)
+		return cmd
 
+	def testPWDhandling(self):
 		# check if PWD at "parse_options" is remembered after changing dir
-		from zim.notebook.info import NotebookInfo
+		cmd = self.get_cmd('./Notes')
+
 		pwd = os.getcwd()
 		self.addCleanup(os.chdir, pwd)
+		os.chdir('/')
 
 		myinfo = NotebookInfo(pwd + '/Notes')
-		os.chdir('/')
 		notebookinfo, page = cmd.get_notebook_argument()
-
 		self.assertEqual(notebookinfo, myinfo)
-		os.chdir(pwd)
+
+	#def testNotebookName(self):
+	#	pass
+
+	def testFile(self):
+		folder = self.setUpFolder(mock=tests.MOCK_ALWAYS_REAL)
+		file = folder.file('Foo.txt')
+		file.touch()
+
+		for arg in (file.path, file.uri):
+			cmd = self.get_cmd(arg)
+			myinfo = NotebookInfo(file.path)
+			notebookinfo, page = cmd.get_notebook_argument()
+			self.assertEqual(notebookinfo, myinfo)
+			notebook, href = cmd.build_notebook()
+			self.assertEqual(notebook.uri, folder.uri)
+			self.assertEqual(href.names, 'Foo')
+
+		# Test override by page argument
+		cmd = self.get_cmd(file.path, 'Bar')
+		notebook, href = cmd.build_notebook()
+		self.assertEqual(notebook.uri, folder.uri)
+		self.assertEqual(href.names, 'Bar')
+
+	def testNotebookZimFile(self):
+		folder = self.setUpFolder(mock=tests.MOCK_ALWAYS_REAL)
+		config = folder.file('notebook.zim')
+		config.touch()
+
+		for arg in (config.path, config.uri):
+			cmd = self.get_cmd(arg)
+			myinfo = NotebookInfo(config.path)
+			notebookinfo, page = cmd.get_notebook_argument()
+			self.assertEqual(notebookinfo, myinfo)
+			notebook, href = cmd.build_notebook()
+			self.assertEqual(notebook.uri, folder.uri)
+			self.assertIsNone(href)
+
+	def testNestedNotebookZimFile(self):
+		# Test specific case where parent folder also is a notebook - as in bug #2189
+		parentfolder = self.setUpFolder(mock=tests.MOCK_ALWAYS_REAL)
+		parentfolder.file('notebook.zim').touch()
+		folder = parentfolder.folder('MyNotebook')
+		config = folder.file('notebook.zim')
+		config.touch()
+
+		for arg in (config.path, config.uri):
+			cmd = self.get_cmd(arg)
+			myinfo = NotebookInfo(config.path)
+			notebookinfo, page = cmd.get_notebook_argument()
+			self.assertEqual(notebookinfo, myinfo)
+			notebook, href = cmd.build_notebook()
+			self.assertEqual(notebook.uri, folder.uri)
+			self.assertIsNone(href)
 
 
-class TestGui(tests.TestCase):
+
+class TestGuiCommand(TestNotebookCommand):
+
+	cmd_class = GuiCommand
+
+
+class TestGuiStart(tests.TestCase):
+
+	## TODO: test default notebook logic when no argument
 
 	def setUp(self):
 		file = ConfigManager.get_config_file('notebooks.list')
@@ -136,9 +199,40 @@ class TestGui(tests.TestCase):
 		self.assertIs(window2, window)
 			# Ensure repeated calling gives unique window
 
-	# TODO
-	# Check default notebook
-	# Check dialog list prompt
+
+class TestGuiListCommand(tests.TestCase):
+
+	# NotebookDialog has it's own't test cases, which also cover the
+	# AddNotebookDialog and prompt_notebook, so here we can mock it and
+	# just test it is called properly from the command class
+
+	def setUp(self):
+		from zim.notebook import NotebookInfo
+
+		self.folder = self.setUpFolder(mock=tests.MOCK_ALWAYS_REAL)
+		self.folder.touch()
+
+		import zim.gui.notebookdialog
+		orig = zim.gui.notebookdialog.prompt_notebook
+		def restore():
+			zim.gui.notebookdialog.prompt_notebook = orig
+		self.addCleanup(restore)
+
+		def mock():
+			return NotebookInfo(self.folder.uri, name='Test')
+
+		zim.gui.notebookdialog.prompt_notebook = mock
+
+	def runTest(self):
+		from zim.gui.mainwindow import MainWindow
+
+		cmd = GuiCommand('gui')
+		cmd.parse_options('--list')
+
+		with tests.WindowContext(MainWindow):
+			with tests.LoggingFilter('zim', 'Exception while loading plugin:'):
+				window = cmd.run()
+				self.addCleanup(window.destroy)
 
 
 class TestManual(tests.TestCase):
