@@ -1,15 +1,13 @@
-# -*- coding: utf-8 -*-
 
 # Copyright 2008-2014 Jaap Karssenberg <jaap.karssenberg@gmail.com>
 
 '''Test cases for the zim.templates module.'''
 
-from __future__ import with_statement
+
 
 import tests
 
-
-from zim.fs import File, Dir, FileNotFoundError
+from zim.newfs import FileNotFoundError
 
 from zim.templates import *
 
@@ -147,7 +145,7 @@ class TestExpressionParser(tests.TestCase):
 				ExpressionParameter('x')
 			)
 		)
-		#~ print '\nEXPRESSION:', expr
+		#~ print('\nEXPRESSION:', expr)
 		self.assertEqual(expr, wanted)
 
 		## Invalid syntaxes
@@ -191,14 +189,14 @@ class TestExpression(tests.TestCase):
 			),
 		])
 
-		result = expr( {
+		result = expr({
 			'foo': 'FOO',
 			'a': {
 				'b': 'BAR'
 			},
 			'n': 1,
-			'addone': ExpressionFunction(lambda a: a+1)
-		} )
+			'addone': ExpressionFunction(lambda a: a + 1)
+		})
 
 		wanted = ['foooo', 'FOO', 'BAR', True, 2]
 		self.assertEqual(result, wanted)
@@ -318,9 +316,9 @@ class TestTemplateBuilderTextBuffer(tests.TestCase):
 
 		self.assertEqual(result, [
 			E('FOO', None, [
-				u'foo',
+				'foo',
 				E('BAR', None, []),
-				u'\n\t\tdus\n',
+				'\n\t\tdus\n',
 				E('BAR', None, []),
 			])
 		])
@@ -473,11 +471,11 @@ class TestTemplateLoopState(tests.TestCase):
 
 			self.assertEqual(loop.size, 3)
 			self.assertEqual(loop.max, 2)
-			self.assertEqual(loop.prev, None if i == 0 else items[i-1])
+			self.assertEqual(loop.prev, None if i == 0 else items[i - 1])
 			self.assertEqual(loop.current, items[i])
-			self.assertEqual(loop.next, None if i == 2 else items[i+1])
+			self.assertEqual(loop.next, None if i == 2 else items[i + 1])
 			self.assertEqual(loop.index, i)
-			self.assertEqual(loop.count, i+1)
+			self.assertEqual(loop.count, i + 1)
 			self.assertEqual(loop.first, True if i == 0 else False)
 			self.assertEqual(loop.last, True if i == 2 else False)
 			self.assertEqual(loop.parity, 'odd' if i % 2 else 'even')
@@ -555,8 +553,9 @@ class TestTemplateProcessor(tests.TestCase):
 		processor.process(lines, TemplateContextDict(context))
 		self.assertEqual(''.join(lines), '1: aaa\n2: bbb\n3: ccc\n')
 
-	def testInclude(self):
-		# test 'INCLUDE',
+	def testIncludeName(self):
+		# test 'INCLUDE name',
+		# parameter "foo" in the context is ignored
 		processor = TemplateProcessor([
 			E('TEMPLATE', None, [
 				E('INCLUDE', {'expr': ExpressionParameter('foo')}),
@@ -567,9 +566,60 @@ class TestTemplateProcessor(tests.TestCase):
 		])
 
 		lines = []
-		processor.process(lines, TemplateContextDict({'foo': 'foo'}))
+		processor.process(lines, TemplateContextDict({'foo': 'bar'}))
 		self.assertEqual(''.join(lines), 'FOO\nFOO\nFOO\n')
 
+	def testIncludeNameExpr(self):
+		# test 'INCLUDE expression' where expression evals to name
+		# parameter "foo" points to block "bar"
+		processor = TemplateProcessor([
+			E('TEMPLATE', None, [
+				E('INCLUDE', {'expr': ExpressionParameter('foo')}),
+				E('INCLUDE', {'expr': ExpressionParameter('foo')}),
+				E('INCLUDE', {'expr': ExpressionParameter('foo')}),
+			]),
+			E('BLOCK', {'name': 'bar'}, 'FOO\n'),
+		])
+
+		lines = []
+		processor.process(lines, TemplateContextDict({'foo': 'bar'}))
+		self.assertEqual(''.join(lines), 'FOO\nFOO\nFOO\n')
+
+	def testIncludePath(self):
+		# test 'INCLUDE path'
+		def parse_included_file_func(path):
+			self.assertEqual(path, 'include.txt')
+			return [E('TEMPLATE', None, [
+				'INCLUDED TEXT\n'
+			])]
+
+		processor = TemplateProcessor([
+			E('TEMPLATE', None, [
+				E('INCLUDE', {'expr': ExpressionLiteral('include.txt')}),
+			]),
+		], parse_included_file_func=parse_included_file_func)
+
+		lines = []
+		processor.process(lines, TemplateContextDict({}))
+		self.assertEqual(''.join(lines), 'INCLUDED TEXT\n')
+
+	def testIncludePathExpr(self):
+		# test 'INCLUDE expression' where expression evals to path
+		def parse_included_file_func(path):
+			self.assertEqual(path, 'include.txt')
+			return [E('TEMPLATE', None, [
+				'INCLUDED TEXT\n'
+			])]
+
+		processor = TemplateProcessor([
+			E('TEMPLATE', None, [
+				E('INCLUDE', {'expr': ExpressionParameter('path')}),
+			]),
+		], parse_included_file_func=parse_included_file_func)
+
+		lines = []
+		processor.process(lines, TemplateContextDict({'path': 'include.txt'}))
+		self.assertEqual(''.join(lines), 'INCLUDED TEXT\n')
 
 
 class TestTemplateList(tests.TestCase):
@@ -581,7 +631,7 @@ class TestTemplateList(tests.TestCase):
 
 		for cat in categories:
 			templates = list_templates(cat)
-			#~ print '>>', cat, templates
+			#~ print('>>', cat, templates)
 			self.assertGreater(len(templates), 0)
 			for name, filename in templates:
 				template = get_template(cat, name)
@@ -618,7 +668,7 @@ class TestTemplateFunctions(tests.TestCase):
 		func = build_template_functions()['range']
 		self.assertIsInstance(func, ExpressionFunction)
 		self.assertEqual(
-			func(1, 10),
+			list(func(1, 10)),
 			[1, 2, 3, 4, 5, 6, 7, 8, 9]
 		)
 
@@ -629,7 +679,7 @@ class TestTemplateFunctions(tests.TestCase):
 		self.assertIsInstance(func, ExpressionFunction)
 		self.assertTrue(func('%Y %m %d'))
 		self.assertEqual(
-			func('%Y %m %d', date(2014, 05, 26)),
+			func('%Y %m %d', date(2014, 5, 26)),
 			'2014 05 26'
 		)
 
@@ -640,7 +690,7 @@ class TestTemplateFunctions(tests.TestCase):
 		self.assertIsInstance(func, ExpressionFunction)
 		self.assertTrue(func('%Y %W'))
 		self.assertEqual(
-			func('%Y %W', date(2014, 05, 26)),
+			func('%Y %W', date(2014, 5, 26)),
 			'2014 22'
 		)
 
@@ -660,8 +710,7 @@ class TestTemplate(tests.TestCase):
 	def runTest(self):
 		from pprint import pprint
 
-		from zim.fs import File
-		file = File('./tests/data/TestTemplate.html')
+		file = tests.TEST_DATA_FOLDER.file('TestTemplate.html')
 
 		templ = Template(file)
 		#~ pprint(templ.parts) # parser output
@@ -699,13 +748,13 @@ class TestTemplate(tests.TestCase):
 			'uri': ExpressionFunction(lambda l: "URL:%s" % l['name']),
 			'anchor': ExpressionFunction(lambda l: "ANCHOR:%s" % l['name']),
 		})
-		#~ print ''.join(output)
+		#~ print(''.join(output))
 
 		# TODO assert something
 
 		### Test empty template OK as well
-		dir = Dir(self.create_tmp_dir())
-		file = dir.file('empty.html')
+		folder = self.setUpFolder(mock=tests.MOCK_ALWAYS_REAL)
+		file = folder.file('empty.html')
 
 		self.assertRaises(FileNotFoundError, Template, file)
 
@@ -714,3 +763,32 @@ class TestTemplate(tests.TestCase):
 		output = []
 		templ.process(output, {})
 		self.assertEqual(output, [])
+
+
+class TestTemplateInclude(tests.TestCase):
+
+	def setUp(self):
+		folder = self.setUpFolder(mock=tests.MOCK_ALWAYS_REAL) # Can be converted to virual after removal of "zim.fs"
+		self.file = folder.file('template.html')
+		include = folder.file('template/include.html')
+		include.write('INCLUDED TEXT [% foo %]')
+		protected = folder.file('passwd.txt')
+		protected.write('FAIL')
+
+	def testInclude(self):
+		# Test inclusion & parsing of template content
+		# see tests in TestTemplateProcessor for tests of syntax variants
+		self.file.write('[% foo="Test" %][% INCLUDE "include.html" %]')
+		templ = Template(self.file)
+		output = []
+		templ.process(output, {})
+		self.assertEqual(output, ['INCLUDED TEXT ', 'Test'])
+
+	def testIncludeFromParentDirNotAllowed(self):
+		# test 'INCLUDE path' does not allow include from ../../ something
+		self.file.write('[% foo="Test" %][% INCLUDE "../passwd.txt" %]')
+		templ = Template(self.file)
+		output = []
+		with tests.LoggingFilter('zim'):
+			templ.process(output, {})
+		self.assertNotIn('FAIL', output)
