@@ -9,10 +9,14 @@ import os
 import re
 
 # Check if we run the correct python version
-try:
-	assert sys.version_info >= (3, 6)
-except:
-	print('zim needs python >= 3.6', file=sys.stderr)
+REQUIRED_MINIMUM_PYTHON_VERSION = (3, 6)
+USED_PYTHON_VERSION = sys.version_info
+if USED_PYTHON_VERSION < REQUIRED_MINIMUM_PYTHON_VERSION:
+	error_message = 'zim needs python >= {major}.{minor}'.format(
+		major=REQUIRED_MINIMUM_PYTHON_VERSION[0],
+		minor=REQUIRED_MINIMUM_PYTHON_VERSION[1],
+	)
+	sys.stderr.write(error_message)
 	sys.exit(1)
 
 
@@ -116,20 +120,17 @@ def main():
 		sys.exit(1)
 
 	# Run the application and handle some exceptions
+	exit_code = 1
 	try:
-		exitcode = zim.main.main(*sys.argv)
-		sys.exit(exitcode)
+		exit_code = zim.main.main(*sys.argv)
 	except zim.main.GetoptError as err:
 		print(sys.argv[0] + ':', err, file=sys.stderr)
-		sys.exit(1)
 	except zim.main.UsageError as err:
 		print(err.msg, file=sys.stderr)
-		sys.exit(1)
 	except KeyboardInterrupt: # e.g. <Ctrl>C while --server
 		print('Interrupt', file=sys.stderr)
-		sys.exit(1)
-	else:
-		sys.exit(0)
+	finally:
+		sys.exit(exit_code)
 
 
 if __name__ == '__main__':
