@@ -1,13 +1,12 @@
-# -*- coding: utf-8 -*-
 
-# Copyright 2008 Jaap Karssenberg <jaap.karssenberg@gmail.com>
+# Copyright 2008-2022 Jaap Karssenberg <jaap.karssenberg@gmail.com>
 
 '''This module handles parsing and dumping input in plain text'''
 
 import re
 
 import zim.parser
-from zim.parser import fix_line_end, Rule
+from zim.parser import fix_unicode_chars, Rule
 
 from zim.formats import *
 from zim.parsing import url_re
@@ -35,20 +34,20 @@ class Parser(ParserClass):
 
 	# TODO parse markdown style headers
 
-	def parse(self, input, partial=False):
-		if not isinstance(input, basestring):
+	def parse(self, input):
+		if not isinstance(input, str):
 			input = ''.join(input)
 
-		if not partial:
-			input = fix_line_end(input)
+		input = fix_unicode_chars(input)
 
 		parser = zim.parser.Parser(
-			Rule(LINK, url_re.r, process=self.parse_url) # FIXME need .r atribute because url_re is a Re object
+			Rule(LINK, url_re.r, process=self.parse_url) # FIXME need .r attribute because url_re is a Re object
 		)
 
-		builder = ParseTreeBuilder(partial=partial)
+		builder = ParseTreeBuilder()
 		builder.start(FORMATTEDTEXT)
-		parser(builder, input)
+		if input:
+			parser(builder, input)
 		builder.end(FORMATTEDTEXT)
 		return builder.get_parsetree()
 
@@ -64,23 +63,26 @@ class Dumper(DumperClass):
 	# readable.
 
 	BULLETS = {
-		UNCHECKED_BOX:	u'[ ]',
-		XCHECKED_BOX:	u'[x]',
-		CHECKED_BOX:	u'[*]',
-		BULLET:			u'*',
+		UNCHECKED_BOX: '[ ]',
+		XCHECKED_BOX: '[x]',
+		CHECKED_BOX: '[*]',
+		MIGRATED_BOX: '[>]',
+		TRANSMIGRATED_BOX: '[<]',
+		BULLET: '*',
 	}
 
 	# No additional formatting for these tags, otherwise copy-pasting
 	# as plain text is no longer plain text
 	TAGS = {
-		EMPHASIS:		('', ''),
-		STRONG:			('', ''),
-		MARK:			('', ''),
-		STRIKE:			('', ''),
-		VERBATIM:		('', ''),
-		TAG:			('', ''),
-		SUBSCRIPT:		('', ''),
-		SUPERSCRIPT:	('', ''),
+		EMPHASIS: ('', ''),
+		STRONG: ('', ''),
+		MARK: ('', ''),
+		STRIKE: ('', ''),
+		VERBATIM: ('', ''),
+		TAG: ('', ''),
+		SUBSCRIPT: ('', ''),
+		SUPERSCRIPT: ('', ''),
+		HEADING: ('', ''),
 	}
 
 	def dump_indent(self, tag, attrib, strings):
@@ -99,16 +101,20 @@ class Dumper(DumperClass):
 	def dump_h(self, tag, attrib, strings):
 		# Markdown style headers
 		level = int(attrib['level'])
-		if level < 1:   level = 1
-		elif level > 5: level = 5
+		if level < 1:
+			level = 1
+		elif level > 5:
+			level = 5
 
 		if level in (1, 2):
 			# setext-style headers for lvl 1 & 2
-			if level == 1: char = '='
-			else: char = '-'
-			heading = u''.join(strings)
-			underline = char * len(heading)
-			return [heading + '\n', underline]
+			if level == 1:
+				char = '='
+			else:
+				char = '-'
+			heading = ''.join(strings)
+			underline = char * len(heading.strip('\n'))
+			return [heading, underline + '\n']
 		else:
 			# atx-style headers for deeper levels
 			tag = '#' * level
@@ -119,11 +125,11 @@ class Dumper(DumperClass):
 		if 'indent' in attrib:
 			# top level list with specified indent
 			prefix = '\t' * int(attrib['indent'])
-			return self.prefix_lines('\t', strings)
-		elif self.context[-1].tag in (BULLETLIST, NUMBEREDLIST):
+			return self.prefix_lines(prefix, strings)
+		elif self.context[-1].tag == LISTITEM:
 			# indent sub list
 			prefix = '\t'
-			return self.prefix_lines('\t', strings)
+			return self.prefix_lines(prefix, strings)
 		else:
 			# top level list, no indent
 			return strings
@@ -147,7 +153,7 @@ class Dumper(DumperClass):
 			iter = self.context[-1].attrib.get('_iter')
 			if not iter:
 				# First item on this level
-				iter = self.context[-1].attrib.get('start', 1)
+				iter = self.context[-1].attrib.get('start', '1')
 			bullet = iter + '.'
 			self.context[-1].attrib['_iter'] = increase_list_iter(iter) or '1'
 		else:
@@ -164,7 +170,11 @@ class Dumper(DumperClass):
 				prefix = int(attrib['indent']) * '\t'
 				bullet = prefix + bullet
 
-		return (bullet, ' ') + tuple(strings) + ('\n',)
+		return (bullet, ' ') + tuple(strings)
+
+	def dump_anchor(self, tag, attrib, strings=None):
+		# TODO: what should be returned here?
+		return ("[id: ", attrib['name'], "]")
 
 	def dump_link(self, tag, attrib, strings=None):
 		# Just plain text, either text of link, or link href
@@ -181,10 +191,8 @@ class Dumper(DumperClass):
 		# Just plain text, either alt text or src
 		src = attrib['src']
 		alt = attrib.get('alt')
-		if alt:
-			return alt
-		else:
-			return src
+		text = alt if alt else src
+		return [text]
 
 	def dump_object_fallback(self, tag, attrib, strings):
 		return strings
@@ -206,23 +214,18 @@ class Dumper(DumperClass):
 			table += [rowline(line) for line in row]
 			table.append(rowsep('-'))
 
-		return map(lambda line: line+"\n", table)
-
-	@staticmethod
-	def _concat(s):
-		return s if isinstance(s, basestring) else ''.join(s)
+		return [line + "\n" for line in table]
 
 	def dump_thead(self, tag, attrib, strings):
-		return [ strings ] # HACK to keep row structure
+		return [strings] # HACK to keep row structure
 
 	def dump_trow(self, tag, attrib, strings):
-		return [ strings ] # HACK to keep row structure
-
-	def dump_th(self, tag, attrib, strings):
-		strings = [s.replace('|', '∣') for s in strings]
-		return [self._concat(strings)]
+		return [strings] # HACK to keep row structure
 
 	def dump_td(self, tag, attrib, strings):
-		strings = [s.replace('|', '∣') for s in strings]
-		return [self._concat(strings)]
+		return [''.join(strings)] # NOTE: no escaping done here !
 
+	dump_th = dump_td
+
+	def dump_line(self, tag, attrib, strings=None):
+		return ('-' * 20) + '\n'
