@@ -8,7 +8,7 @@ import logging
 
 logger = logging.getLogger('zim.formats.wiki')
 
-from zim.parser import Rule, fix_line_end, convert_space_to_tab
+from zim.parser import Rule, fix_unicode_chars, convert_space_to_tab
 from zim.parser import Parser as RuleParser
 from zim.parsing import url_encode, URL_ENCODE_DATA, \
 	escape_string, unescape_string, split_escaped_string
@@ -45,21 +45,11 @@ bullet_pattern = '(?:[\\*\u2022]|\\[[ \\*x><]\\]|\\d+\\.|[a-zA-Z]\\.)[\\ \\t]+'
 	# and '[ ]', '[*]', '[x]', '[>]' or [<]' for checkbox items
 	# and '1.', '10.', or 'a.' for numbered items (but not 'aa.')
 
-bullet_line_re = re.compile(r'^(\t*)(%s)(.*\n)$' % bullet_pattern)
+bullet_line_re = re.compile(r'^(\t*)(%s)(.*$\n?)' % bullet_pattern)
 	# matches list item: prefix, bullet, text
 
 number_bullet_re = re.compile('^(\d+|[a-zA-Z])\.$')
 
-def check_number_bullet(bullet):
-	'''If bullet is a numbered bullet this returns the number or letter,
-	C{None} otherwise
-	'''
-	m = number_bullet_re.match(bullet)
-	if m:
-		return m.group(1)
-	else:
-		return None
-
 param_re = re.compile('([\w-]+)=("(?:[^"]|"{2})*"|\S*)')
 	# matches parameter list for objects
 	# allow name="foo bar" and name=Foo
@@ -103,15 +93,16 @@ _file = r'''
 	'''
 
 url_re = re.compile(
-	f'''\\b
-	(?!__)(?P<url>{_url})     |
-	(?!__)(?P<email>{_email}) |
-	(?P<fileuri>{_file})
-	''',
+	'''\\b
+	(?!__)(?P<url>%s)     |
+	(?!__)(?P<email>%s) |
+	(?P<fileuri>%s)
+	''' % (_url, _email, _file),
 	re.VERBOSE
 )
 
-url_trailing_punctuation = ('?', '!', '.', ',', ':', '*', '_', '~')
+url_trailing_punctuation = ('?', '!', '.', ',', ':', '*', '_', '~', "'", '"')
+	# do not add ";" here, it is handled separatedly in the function
 
 
 def is_url(text):
@@ -198,7 +189,8 @@ class WikiParser(object):
 
 	def __call__(self, builder, text):
 		builder.start(FORMATTEDTEXT)
-		self.block_parser(builder, text)
+		if text:
+			self.block_parser(builder, text)
 		builder.end(FORMATTEDTEXT)
 
 	def _init_inline_parse(self):
@@ -241,9 +233,9 @@ class WikiParser(object):
 			Rule(
 				'X-Bullet-List',
 				r'''(
-					^ %s .* \n								# Line starting with bullet
+					^ %s .* $\n?							# Line starting with bullet
 					(?:
-						^ \t* %s .* \n						# Line with same or more indent and bullet
+						^ \t* %s .* $\n?					# Line with same or more indent and bullet
 					)*										# .. repeat
 				)''' % (bullet_pattern, bullet_pattern),
 				process=self.parse_list
@@ -251,9 +243,9 @@ class WikiParser(object):
 			Rule(
 				'X-Indented-Bullet-List',
 				r'''(
-					^(?P<list_indent>\t+) %s .* \n			# Line with indent and bullet
+					^(?P<list_indent>\t+) %s .* $\n?		# Line with indent and bullet
 					(?:
-						^(?P=list_indent) \t* %s .* \n		# Line with same or more indent and bullet
+						^(?P=list_indent) \t* %s .* $\n?	# Line with same or more indent and bullet
 					)*										# .. repeat
 				)''' % (bullet_pattern, bullet_pattern),
 				process=self.parse_list
@@ -261,10 +253,10 @@ class WikiParser(object):
 			Rule(
 				'X-Indented-Block',
 				r'''(
-					^(?P<block_indent>\t+) .* \n			# Line with indent
+					^(?P<block_indent>\t+) .* $\n?			 # Line with indent
 					(?:
-						^(?P=block_indent) (?!\t|%s) .* \n	# Line with _same_ indent, no bullet
-					)*										# .. repeat
+						^(?P=block_indent) (?!\t|%s) .* $\n? # Line with _same_ indent, no bullet
+					)*										 # .. repeat
 				)''' % bullet_pattern,
 				process=self.parse_indent
 			),
@@ -290,7 +282,7 @@ class WikiParser(object):
 				process=self.parse_object
 			),
 			Rule(HEADING,
-				r'^( ==+ [\ \t]+ \S.*? ) [\ \t]* =* \n',		# "==== heading ===="
+				r'^( ==+ [\ \t]+ \S.*? ) [\ \t]* =* $\n?',		# "==== heading ===="
 				process=self.parse_heading
 			),
 			# standard table format
@@ -302,7 +294,7 @@ class WikiParser(object):
 				process=self.parse_table
 			),
 			# line format
-			Rule(LINE, r'(?<=\n)-{5,}(?=\n)', process=self.parse_line) # \n----\n
+			Rule(LINE, r'(?<=\n)-{5,}\n', process=self.parse_line) # \n----\n
 
 		)
 		p.process_unmatched = self.parse_para
@@ -346,7 +338,7 @@ class WikiParser(object):
 			# Special case to ensure backward compatibility for versions where
 			# tables could be stored as objects
 			if param.strip() != '':
-				logger.warn('Table object had unexpected parameters: %s', param.strip())
+				logger.warning('Table object had unexpected parameters: %s', param.strip())
 			lines = body.splitlines(True)
 			headerrow = lines[0]
 			alignstyle = lines[1]
@@ -486,69 +478,73 @@ class WikiParser(object):
 		'''
 		if indent:
 			text = _remove_indent(text, indent)
-			attrib = {'indent': len(indent)}
+			indent = len(indent)
 		else:
 			attrib = None
 
 		lines = text.splitlines(True)
-		self.parse_list_lines(builder, lines, 0, attrib)
+		self.parse_list_lines(builder, lines, indent)
 
-	def parse_list_lines(self, builder, lines, level, attrib=None):
-		listtype = None
-		first = True
-		while lines:
-			line = lines[0]
+	def parse_list_lines(self, builder, lines, list_indent=None):
+		stack = [(None, -1)] # list type, indent
+
+		def start_list(number_m):
+			attrib = {'indent': list_indent} if (list_indent and len(stack) == 1) else None
+			if number_m:
+				l = NUMBEREDLIST
+				attrib = attrib or {}
+				attrib['start'] = number_m.group(1)
+			else:
+				l = BULLETLIST
+			builder.start(l, attrib)
+			stack.append((l, my_indent))
+
+		for line in lines:
 			m = bullet_line_re.match(line)
 			assert m, 'Line does not match a list item: >>%s<<' % line
 			prefix, bullet, text = m.groups()
+			my_indent = len(prefix)
 			bullet = bullet.rstrip()
+			number_m = number_bullet_re.match(bullet)
 
-			if first:
-				number = check_number_bullet(bullet)
-				if number:
-					listtype = NUMBEREDLIST
-					if not attrib:
-						attrib = {}
-					attrib['start'] = number
-				else:
-					listtype = BULLETLIST
-				builder.start(listtype, attrib)
-				first = False
-
-			mylevel = len(prefix)
-			if mylevel > level:
-				self.parse_list_lines(builder, lines, level + 1) # recurs
-			elif mylevel < level:
-				builder.end(listtype)
-				return
+			if my_indent > stack[-1][-1]:
+				start_list(number_m)
+			elif my_indent <= stack[-2][-1]:
+				# Check parent of current level my_indent could be lower than
+				# current level but still on same hierarchy level due to
+				# inconsistent formatting
+				while my_indent <= stack[-2][-1]:
+					l, i = stack.pop()
+					builder.end(l)
+			elif (stack[-1][0] == NUMBEREDLIST and bullet in self.BULLETS) \
+				or (stack[-1][0] == BULLETLIST and number_m):
+					# inconsistent list, break current level and start new list
+					l, x = stack.pop()
+					builder.end(l)
+					start_list(number_m)
 			else:
-				if listtype == NUMBEREDLIST:
-					if bullet in self.BULLETS:
-						builder.end(listtype)
-						return self.parse_list_lines(builder, lines, level) # recurs
-					else:
-						attrib = None
-				else: # BULLETLIST
-					if bullet in self.BULLETS:
-						attrib = {'bullet': self.BULLETS[bullet]}
-					elif number_bullet_re.match(bullet):
-						builder.end(listtype)
-						return self.parse_list_lines(builder, lines, level) # recurs
-					else:
-						attrib = {'bullet': BULLET}
-				builder.start(LISTITEM, attrib)
+				pass # we are at right level
+
+			if stack[-1][0] == NUMBEREDLIST:
+				attrib = None
+			else: # BULLETLIST
+				attrib = {'bullet': self.BULLETS.get(bullet, BULLET)}
+
+			builder.start(LISTITEM, attrib)
+			if text: # Might be empty line apart from bullet - even no newline at end of buffer
 				self.inline_parser(builder, text)
-				builder.end(LISTITEM)
+			builder.end(LISTITEM)
 
-				lines.pop(0)
-
-		builder.end(listtype)
+		while len(stack) > 1:
+			l, x = stack.pop()
+			builder.end(l)
 
 	def parse_indent(self, builder, text, indent):
 		'''Parse indented blocks and turn them into 'div' elements'''
 		text = _remove_indent(text, indent)
 		builder.start(BLOCK, {'indent': len(indent)})
-		self.inline_parser(builder, text)
+		if text: # Might be empty line apart from indent characters - even no newline at end of buffer
+			self.inline_parser(builder, text)
 		builder.end(BLOCK)
 
 	def parse_link(self, builder, text):
@@ -629,12 +625,12 @@ class Parser(ParserClass):
 	def __init__(self, version=WIKI_FORMAT_VERSION):
 		self.version = version
 
-	def parse(self, input, partial=False, file_input=False):
+	def parse(self, input, file_input=False):
 		if not isinstance(input, str):
 			input = ''.join(input)
 
-		if not partial:
-			input = fix_line_end(input)
+		input = input.replace('\u2029', ' ') # Unicode PARAGRAPH SEPARATOR, causes conflict between "splitlines()" and regex "\n" matching - see issues #1760
+		input = fix_unicode_chars(input)
 
 		meta, version = None, False
 		if file_input:
@@ -650,7 +646,7 @@ class Parser(ParserClass):
 		else:
 			mywikiparser = WikiParser(backward_indented_blocks=True, backward_url_parsing=True)
 
-		builder = ParseTreeBuilder(partial=partial)
+		builder = ParseTreeBuilder()
 		mywikiparser(builder, input)
 
 		parsetree = builder.get_parsetree()
@@ -696,14 +692,18 @@ class Dumper(TextDumper):
 				('Content-Type', 'text/x-zim-wiki'),
 				('Wiki-Format', WIKI_FORMAT_VERSION),
 			)
-			return [dump_header_lines(header, getattr(tree, 'meta', {})), '\n'] \
-						+ TextDumper.dump(self, tree)
+			body = TextDumper.dump(self, tree)
+			if not body[-1].endswith('\n'):
+				body[-1] = body[-1] + '\n'
+			return [dump_header_lines(header, getattr(tree, 'meta', {})), '\n'] + body
 		else:
 			return TextDumper.dump(self, tree)
 
 	def dump_pre(self, tag, attrib, strings):
 		# Indent and wrap with "'''" lines
 		strings.insert(0, "'''\n")
+		if strings and not strings[-1].endswith('\n'):
+			strings[-1] = strings[-1] + '\n'
 		strings.append("'''\n")
 		strings = self.dump_indent(tag, attrib, strings)
 		return strings
@@ -717,7 +717,8 @@ class Dumper(TextDumper):
 			level = 5
 		tag = '=' * (7 - level)
 		strings.insert(0, tag + ' ')
-		strings.append(' ' + tag)
+		text = strings.pop()
+		strings.append(text.rstrip() + ' ' + tag + '\n') # includes stripping "\n"
 		return strings
 
 	def dump_anchor(self, tag, attrib, strings=None):
