@@ -15,13 +15,13 @@ from gi.repository import GdkPixbuf
 import logging
 
 
-from zim.fs import File, Dir, FS
-from zim.newfs import LocalFolder
+from zim.fs import adapt_from_oldfs
+from zim.newfs import FilePath, LocalFile, LocalFolder
 from zim.notebook import Path, HRef, set_parsetree_attributes_to_resolve_links, \
 	replace_parsetree_links_and_copy_images
 from zim.parsing import is_url_re, url_encode, link_type, URL_ENCODE_READABLE
 from zim.formats import get_format, ParseTree, ParseTreeBuilder, \
-	FORMATTEDTEXT, IMAGE, LINK, TAG, OBJECT, VisitorSkip
+	FORMATTEDTEXT, IMAGE, LINK, TAG, OBJECT
 from zim.export.linker import StaticExportLinker
 
 
@@ -146,7 +146,6 @@ def serialize_parse_tree(register_buf, content_buf, start, end, user_data):
 def deserialize_parse_tree(register_buf, content_buf, iter, data, length, create_tags, user_data):
 	notebook, path = user_data
 	tree = ParseTree().fromstring(data)
-	tree.resolve_images(notebook, path)
 	content_buf.insert_parsetree(iter, tree, interactive=True)
 	return True
 
@@ -185,8 +184,8 @@ def deserialize_image(register_buf, content_buf, iter, data, length, create_tags
 
 	file = dir.new_file('pasted_image.%s' % extension)
 	logger.debug("Saving image from clipboard to %s", file)
+	file.touch() # notify version control
 	pixbuf.savev(file.path, format, [], [])
-	FS.emit('path-created', file) # notify version control
 
 	# and insert it in the page
 	links = [file.uri]
@@ -225,8 +224,7 @@ def parsetree_from_selectiondata(selectiondata, notebook, path=None, text_format
 	targetname = selectiondata.get_target().name()
 	if targetname == PARSETREE_TARGET_NAME:
 		parsetree = ParseTree().fromstring(selectiondata.get_data())
-		replace_parsetree_links_and_copy_images(parsetree, notebook, path)
-		return parsetree
+		return replace_parsetree_links_and_copy_images(parsetree, notebook, path)
 	elif targetname == PAGELIST_TARGET_NAME or targetname in URI_TARGET_NAMES:
 		links = selectiondata.get_uris()
 		return _link_tree(links, notebook, path)
@@ -241,7 +239,7 @@ def parsetree_from_selectiondata(selectiondata, notebook, path=None, text_format
 					tag_name = 'pre' if '\n' in text else 'code'
 				else:
 					tag_name = text_format[9:]
-				builder = ParseTreeBuilder(partial=True)
+				builder = ParseTreeBuilder()
 				builder.start('zim-tree', {})
 				builder.start(tag_name, {})
 				builder.text(text)
@@ -249,7 +247,7 @@ def parsetree_from_selectiondata(selectiondata, notebook, path=None, text_format
 				builder.end('zim-tree')
 				return builder.get_parsetree()
 			else:
-				return get_format(text_format).Parser().parse(text, partial=True)
+				return get_format(text_format).Parser().parse(text)
 		else:
 			return None
 	elif targetname in IMAGE_TARGET_NAMES:
@@ -274,8 +272,8 @@ def parsetree_from_selectiondata(selectiondata, notebook, path=None, text_format
 
 		file = dir.new_file('pasted_image.%s' % extension)
 		logger.debug("Saving image from clipboard to %s", file)
+		file.touch() # notify version control
 		pixbuf.savev(file.path, format, [], [])
-		FS.emit('path-created', file) # notify version control
 
 		links = [file.uri]
 		return _link_tree(links, notebook, path)
@@ -285,6 +283,7 @@ def parsetree_from_selectiondata(selectiondata, notebook, path=None, text_format
 
 def _link_tree(links, notebook, path):
 	# Convert a list of links (of any type) into a parsetree
+	# To be used on the recieving end of a copy-paste, arguments are the target notebook and path
 	#~ print('LINKS: ', links)
 	#~ print('NOTEBOOK and PATH:', notebook, path)
 	builder = ParseTreeBuilder()
@@ -303,10 +302,11 @@ def _link_tree(links, notebook, path):
 				type = link_type(link)
 		elif type == 'file':
 			try:
-				file = File(link)
-				isimage = file.isimage()
-			except:
+				file = LocalFile(link)
+			except ValueError:
 				pass
+			else:
+				isimage = file.isimage()
 
 		logger.debug('Pasting link: %s (type: %s, isimage: %s)', link, type, isimage)
 
@@ -327,18 +327,18 @@ def _link_tree(links, notebook, path):
 				href.anchor = anchor
 				link = href.to_wiki_link()
 				if notebook.config['Notebook']['short_links']:
-					name = href.parts()[-1]
-					if anchor:
-						name += '#' + anchor
+					name = href.short_name()
 			elif type == 'file':
-				file = File(link) # Assume links are always URIs
-				link = notebook.relative_filepath(file, path) or file.uri
+				try:
+					file = LocalFile(link) # Assume links are always URIs
+					link = notebook.relative_filepath(file, path) or file.uri
+				except:
+					pass
 
 			builder.append(LINK, {'href': link}, name or link)
 
 	builder.end(FORMATTEDTEXT)
 	tree = builder.get_parsetree()
-	tree.resolve_images(notebook, path)
 	tree.decode_urls()
 	return tree
 
@@ -412,9 +412,10 @@ class UriData(ClipboardData):
 		uris = []
 		text = []
 		for o in obj:
-			if isinstance(o, (File, Dir)):
+			o = adapt_from_oldfs(o)
+			if isinstance(o, FilePath):
 				uris.append(o.uri)
-				text.append(o.user_path or o.path)
+				text.append(o.userpath)
 			else:
 				uri = o if isinstance(o, str) else o.uri
 				uris.append(uri)
@@ -434,15 +435,27 @@ class InterWikiLinkData(UriData):
 	targets = (PARSETREE_TARGET,) + UriData.targets
 
 	def __init__(self, href, url):
-		UriData.__init__(self, url)
 		self.interwiki_href = href
+		self.interwiki_url = url
+		if self.interwiki_url is not None:
+			UriData.__init__(self, url)
+		else:
+			pass # interwiki may be undefined, resulting in url being None
 
 	def get_data_as(self, targetid):
 		if targetid == PARSETREE_TARGET_ID:
-			parsetree = _link_tree((self.interwiki_href,), None, None)
+			builder = ParseTreeBuilder()
+			builder.start(FORMATTEDTEXT)
+			builder.append(LINK, {'href': self.interwiki_href}, self.interwiki_href)
+			builder.end(FORMATTEDTEXT)
+			parsetree = builder.get_parsetree()
+			parsetree._set_root_attrib('notebook', '-') # force resolve on paste
+			parsetree._set_root_attrib('page', '-')
 			return parsetree.tostring()
-		else:
+		elif self.interwiki_url is not None:
 			return UriData.get_data_as(self, targetid)
+		else:
+			return self.interwiki_href
 
 
 class ParseTreeData(ClipboardData):
@@ -457,8 +470,8 @@ class ParseTreeData(ClipboardData):
 
 	def get_data_as(self, targetid):
 		if targetid == PARSETREE_TARGET_ID:
-			set_parsetree_attributes_to_resolve_links(self.parsetree, self.notebook, self.path)
-			return self.parsetree.tostring()
+			newtree = set_parsetree_attributes_to_resolve_links(self.parsetree, self.notebook, self.path)
+			return newtree.tostring()
 		elif targetid == HTML_TARGET_ID:
 			dumper = get_format('html').Dumper(
 				linker=StaticExportLinker(self.notebook, source=self.path))
@@ -506,9 +519,7 @@ class PageLinkData(ClipboardData):
 				text = self.text
 			elif self.notebook.config['Notebook']['short_links']:
 				href = HRef.new_from_wiki_link(link)
-				text = href.parts()[-1]
-				if self.anchor:
-					text += '#' + self.anchor
+				text = href.short_name()
 			else:
 				text = link
 
@@ -653,10 +664,10 @@ class ClipboardManager(object):
 			if selectiondata:
 				return parsetree_from_selectiondata(selectiondata, notebook, path, text_format)
 			else:
-				logger.warn('Did not get requested data from clipboard')
+				logger.warning('Did not get requested data from clipboard')
 				return None
 		else:
-			logger.warn('Could not paste - no compatible data types on clipboard')
+			logger.warning('Could not paste - no compatible data types on clipboard')
 			return None
 
 	def set_pagelink(self, notebook, path, anchor=None, text=None):
