@@ -1,23 +1,23 @@
-# -*- coding: utf-8 -*-
 
-# Copyright 2008,2014 Jaap Karssenberg <jaap.karssenberg@gmail.com>
+# Copyright 2008-2017 Jaap Karssenberg <jaap.karssenberg@gmail.com>
 
-from __future__ import with_statement
 
-import gtk
+
+from gi.repository import Gtk
 
 import logging
 
 import zim.formats
 import zim.templates
 
-from zim.fs import File, Dir, TmpFile
+from zim.newfs import LocalFile, LocalFolder, TmpFile
 from zim.notebook import Path, encode_filename
 from zim.notebook.operations import NotebookOperation
 
 from zim.gui.widgets import Assistant, AssistantPage, \
 	ProgressDialog, ErrorDialog, QuestionDialog, \
-	MessageDialog, LogFileDialog, Button
+	MessageDialog, LogFileDialog
+from zim.gui.applications import open_file
 
 from zim.export import *
 from zim.export.selections import *
@@ -28,11 +28,12 @@ logger = logging.getLogger('zim.export')
 
 class ExportDialog(Assistant):
 
-	def __init__(self, ui):
-		Assistant.__init__(self, ui, _('Export'), # T: dialog title
+	def __init__(self, widget, notebook, page=None):
+		Assistant.__init__(self, widget, _('Export'), # T: dialog title
 			help=':Help:Export', defaultwindowsize=(400, 325))
+		self.notebook = notebook
 
-		self.append_page(InputPage(self))
+		self.append_page(InputPage(self, page))
 		self.append_page(FormatPage(self))
 		self.append_page(OutputPage(self))
 
@@ -50,7 +51,7 @@ class ExportDialog(Assistant):
 		logging_context = LogContext()
 		with logging_context:
 			op = NotebookOperation(
-				self.ui.notebook,
+				self.notebook,
 				_('Exporting notebook'), # T: Title for progressbar window
 				exporter.export_iter(selection)
 			)
@@ -60,23 +61,23 @@ class ExportDialog(Assistant):
 		if op.cancelled:
 			return False
 		else:
-			#~ print '>>> %s E: %i, W: %i' % (
+			#~ print('>>> %s E: %i, W: %i' % ()
 				#~ logging_context.file.path,
 				#~ logging_context.handler.n_error, logging_context.handler.n_warning)
 			#~ print logging_context.file.read()
-			#~ print '---'
+			#~ print('---')
 			ExportDoneDialog(self, logging_context, output).run()
 			return True
 
 	def get_selection(self):
 		if self.uistate['selection'] == 'all':
-			return AllPages(self.ui.notebook)
+			return AllPages(self.notebook)
 		else:
 			path = self.uistate['selected_page']
 			if self.uistate['selection_recursive']:
-				return SubPages(self.ui.notebook, path)
+				return SubPages(self.notebook, path)
 			else:
-				return SinglePage(self.ui.notebook, path)
+				return SinglePage(self.notebook, path)
 
 	def get_exporter(self):
 		#~ import pprint
@@ -134,8 +135,8 @@ class ExportDialog(Assistant):
 				return None, None
 
 	def get_folder(self):
-		dir = Dir(self.uistate['output_folder'])
-		if dir.exists() and len(dir.list()) > 0:
+		dir = LocalFolder(self.uistate['output_folder'])
+		if dir.exists() and len(dir.list_names()) > 0:
 			ok = QuestionDialog(self, (
 				_('Folder exists: %s') % dir.path, # T: message heading
 				_('Folder already exists and has content, '
@@ -148,7 +149,7 @@ class ExportDialog(Assistant):
 		return dir
 
 	def get_file(self):
-		file = File(self.uistate['output_file'])
+		file = LocalFile(self.uistate['output_file'])
 		if file.exists():
 			ok = QuestionDialog(self, (
 				_('File exists'), # T: message heading
@@ -165,7 +166,7 @@ class InputPage(AssistantPage):
 
 	title = _('Select the pages to export') # T: title of step in export dialog
 
-	def __init__(self, assistant):
+	def __init__(self, assistant, page=None):
 		AssistantPage.__init__(self, assistant)
 
 		self.add_form((
@@ -180,7 +181,7 @@ class InputPage(AssistantPage):
 			('page', 'page', _('Page')), # T: Input field in export dialog
 			('recursive', 'bool', _('Include subpages')), # T: Input field in export dialog
 		), {
-			'page': assistant.ui.page,
+			'page': page,
 			'recursive': True,
 		},
 		depends={
@@ -231,13 +232,11 @@ class FormatPage(AssistantPage):
 		})
 
 		## Same button appears in edit preferences dialog
-		if gtk.gtk_version >= (2, 10) \
-		and gtk.pygtk_version >= (2, 10):
-			url_button = gtk.LinkButton(
-				'https://github.com/jaap-karssenberg/zim-wiki/wiki/Templates',
-				_('Get more templates online') # T: label for button with URL
-			)
-			self.pack_start(url_button, False)
+		url_button = Gtk.LinkButton(
+			'https://github.com/jaap-karssenberg/zim-wiki/wiki/Templates',
+			_('Get more templates online') # T: label for button with URL
+		)
+		self.pack_start(url_button, False, True, 0)
 
 
 		# Set template list based on selected format
@@ -255,14 +254,13 @@ class FormatPage(AssistantPage):
 			combobox.set_sensitive(True)
 
 			template = self.uistate['template']
-			if template == '__file__':
-				# Select "Other..."
-				combobox.set_active(len(templates))
-			else:
-				try:
+			try:
+				if template == '__file__':
+					self.form['template'] = self.CHOICE_OTHER
+				else:
 					self.form['template'] = template
-				except ValueError:
-					combobox.set_active(0)
+			except ValueError:
+				combobox.set_active(0)
 
 		self.form.widgets['format'].connect_object('changed', set_templates, self)
 
@@ -273,7 +271,7 @@ class FormatPage(AssistantPage):
 							o.get_active_text() == self.CHOICE_OTHER))
 
 		# Check if we have a document root - if not disable all options
-		docroot = assistant.ui.notebook.document_root
+		docroot = assistant.notebook.document_root
 		if not docroot:
 			for widget in self.form.widgets:
 				if widget.startswith('document_root:'):
@@ -284,7 +282,7 @@ class FormatPage(AssistantPage):
 		self.uistate.setdefault('format', 'HTML')
 		self.uistate.setdefault('template', 'Default')
 		self.uistate.setdefault('template_file', '')
-		self.uistate.setdefault('document_root', 'absolute', check=set(('absolute', 'url')))
+		self.uistate.setdefault('document_root', 'absolute', check={'absolute', 'url'})
 		self.uistate.setdefault('document_root_url', '')
 
 		try:
@@ -341,9 +339,9 @@ class OutputPage(AssistantPage):
 		# on whether we selected full notebook or single page in the
 		# first page
 		self.uistate.setdefault('output', 'multi_file')
-		self.uistate.setdefault('output_folder', None, Dir)
+		self.uistate.setdefault('output_folder', None, LocalFolder)
 		self.uistate.setdefault('index_page', '')
-		self.uistate.setdefault('output_file', None, File)
+		self.uistate.setdefault('output_file', None, LocalFile)
 
 		if self.uistate.get('format', '').startswith('MHTML'):
 			# XXX make this a format property to be queried
@@ -370,11 +368,11 @@ class OutputPage(AssistantPage):
 			ext = zim.formats.get_format(format).info['extension']
 
 		if self.uistate['output_file'] \
-		and isinstance(self.uistate['output_file'], File):
-			dir = self.uistate['output_file'].dir
+		and isinstance(self.uistate['output_file'], LocalFile):
+			dir = self.uistate['output_file'].parent()
 			file = dir.file(encode_filename(basename + '.' + ext))
 		else:
-			file = File('~/' + encode_filename(basename + '.' + ext))
+			file = LocalFile('~/' + encode_filename(basename + '.' + ext))
 		self.uistate['output_file'] = file
 
 		self.form['file'] = self.uistate['output_file']
@@ -428,7 +426,7 @@ class ExportDoneDialog(MessageDialog):
 		MessageDialog.__init__(self, parent, (_('Export completed'), text))
 			# T: label in export dialog
 
-		log_button = Button(_('View _Log'), stock='gtk-file')
+		log_button = Gtk.Button.new_with_mnemonic(_('View _Log'))
 			# T: button in export dialog
 		log_button.set_sensitive(logging_context.file.exists())
 		log_button.connect_object(
@@ -443,7 +441,7 @@ class ExportDoneDialog(MessageDialog):
 		LogFileDialog(self, self.logging_context.file).run()
 
 	def on_open_file(self):
-		self.ui.open_file(self.output) # XXX
+		open_file(self.output)
 
 
 class LogContext(object):
@@ -457,6 +455,7 @@ class LogContext(object):
 		self.level = level
 		self.file = TmpFile(basename='export-log.txt', unique=False, persistent=True)
 		self.file.remove() # clean up previous run
+		self.file.parent().touch()
 		self.handler = LogHandler(self.file.path)
 		self.handler.setLevel(self.level)
 		self.handler.addFilter(LogFilter(names))
