@@ -10,7 +10,7 @@ from functools import partial
 from . import Path, HRef
 
 from zim.newfs import LocalFile, SEP
-from zim.formats import IMAGE, LINK, OBJECT, VisitorSkip
+from zim.formats import IMAGE, LINK, OBJECT, TEXT
 from zim.parsing import link_type
 
 
@@ -21,51 +21,48 @@ def update_parsetree_and_copy_images(parsetree, notebook, oldpath, newpath):
 	'''Convenience method to update parsetree in place and copy any images
 	for use in a new page
 	'''
-	set_parsetree_attributes_to_resolve_links(parsetree, notebook, oldpath)
-	replace_parsetree_links_and_copy_images(parsetree, notebook, newpath)
+	newtree = set_parsetree_attributes_to_resolve_links(parsetree, notebook, oldpath)
+	return replace_parsetree_links_and_copy_images(newtree, notebook, newpath)
 
 
 def set_parsetree_attributes_to_resolve_links(parsetree, notebook, path):
 	'''Changes parsetree in place so links can later be resolved against
 	the source page
 	'''
-	parsetree._set_root_attrib('notebook', notebook.interwiki)
-	parsetree._set_root_attrib('page', path.name)
-	parsetree.replace(
+	newtree = parsetree.substitute_elements(
 		(LINK, IMAGE, OBJECT),
 		partial(_resolve_links_and_images, notebook, path)
 	)
+	newtree._set_root_attrib('notebook', notebook.interwiki)
+	newtree._set_root_attrib('page', path.name)
+	return newtree
 
 
 def _resolve_links_and_images(notebook, src_path, node):
 	if node.tag == LINK:
-		href = node.get('href')
+		href = node.attrib['href']
 		my_type = link_type(href)
 		if my_type == 'page':
 			target = notebook.pages.resolve_link(src_path, HRef.new_from_wiki_link(href))
-			node.set('_href', target.name)
+			node.attrib['_href'] = target.name
 		elif my_type == 'file':
 			try:
 				target = notebook.resolve_file(href, src_path)
 			except:
 				pass # may by e.g. malformed path
 			else:
-				node.set('_href', target.uri)
-		return node
+				node.attrib['_href'] = target.uri
 	elif node.tag == IMAGE:
-		target = notebook.resolve_file(node.get('src'), src_path)
-		node.set('_src', target.uri)
-		return node
-	elif node.tag == OBJECT:
-		if node.get('type').startswith('image+'):
-			# Objects based on generated images
-			target = notebook.resolve_file(node.get('src'), src_path)
-			node.set('_src', target.uri)
-			return node
-		else:
-			raise VisitorSkip
+		target = notebook.resolve_file(node.attrib['src'], src_path)
+		node.attrib['_src'] = target.uri
+	elif node.tag == OBJECT and node.attrib['type'].startswith('image+'):
+		# Objects based on generated images
+		target = notebook.resolve_file(node.attrib['src'], src_path)
+		node.attrib['_src'] = target.uri
 	else:
-		raise AssertionError('unknown tag')
+		pass
+
+	return node
 
 
 def replace_parsetree_links_and_copy_images(parsetree, notebook, path):
@@ -83,7 +80,7 @@ def replace_parsetree_links_and_copy_images(parsetree, notebook, path):
 	src_page = parsetree._pop_root_attrib('page')
 	if src_notebook and src_notebook != notebook.interwiki:
 		# Update for cross notebook link
-		parsetree.replace(
+		return parsetree.substitute_elements(
 			(LINK, IMAGE, OBJECT),
 			partial(_replace_links_to_interwiki_and_copy_images, src_notebook, notebook, path)
 		)
@@ -91,13 +88,13 @@ def replace_parsetree_links_and_copy_images(parsetree, notebook, path):
 		# Update to new page
 		old_page = notebook.get_page(Path(src_page))
 		old_folder = old_page.attachments_folder
-		parsetree.replace(
+		return parsetree.substitute_elements(
 			(LINK, IMAGE, OBJECT),
 			partial(_replace_links_to_page_and_copy_images, notebook, old_folder, path)
 		)
 	else:
 		# Leave links alone, remove "_href" attribs
-		parsetree.replace(
+		return parsetree.substitute_elements(
 			(LINK, IMAGE, OBJECT),
 			_strip_link_and_image_attribs
 		)
@@ -106,13 +103,11 @@ def replace_parsetree_links_and_copy_images(parsetree, notebook, path):
 def _strip_link_and_image_attribs(node):
 	if node.tag == LINK:
 		node.attrib.pop('_href', None)
-		return node
 	elif node.tag == IMAGE \
-		or (node.tag == OBJECT and node.get('type').startswith('image+')):
+		or (node.tag == OBJECT and node.attrib['type'].startswith('image+')):
 			node.attrib.pop('_src', None)
-			return node
-	else:
-		raise VisitorSkip
+
+	return node
 
 
 def _replace_links_to_interwiki_and_copy_images(src_interwiki, notebook, new_path, node):
@@ -121,7 +116,7 @@ def _replace_links_to_interwiki_and_copy_images(src_interwiki, notebook, new_pat
 		if abs_href:
 			my_type = link_type(abs_href)
 			if my_type == 'page':
-				oldhref = HRef.new_from_wiki_link(node.get('href')) # *not* abs_href
+				oldhref = HRef.new_from_wiki_link(node.attrib['href']) # *not* abs_href
 				new_href = src_interwiki + '?' + abs_href
 				new_href += '#' + oldhref.anchor if oldhref.anchor else ''
 			elif my_type == 'file':
@@ -129,32 +124,25 @@ def _replace_links_to_interwiki_and_copy_images(src_interwiki, notebook, new_pat
 				#       should be improved - e.g. path:./file style links like in docuwiki
 				new_href = abs_href
 			else:
-				logger.warn('Could not update link of type "%s": %s', my_type, abs_href)
-				raise VisitorSkip
+				logger.warning('Could not update link of type "%s": %s', my_type, abs_href)
+				return node
 
-			if node.gettext() == node.get('href'): # *not* abs_href
-				node[:] = [new_href]
-			node.set('href', new_href)
-			return node
-		else:
-			raise VisitorSkip
+			if node.content == [(TEXT, node.attrib['href'])]: # *not* abs_href
+				node.content[:] = [(TEXT, new_href)]
+			node.attrib['href'] = new_href
 	elif node.tag == IMAGE:
 		# Just copy all images - image links to other notebook don't make sense
 		abs_src = node.attrib.pop('_src', None)
 		if abs_src:
 			src_file = LocalFile(abs_src)
 			return _copy_image(notebook, new_path, src_file, node)
-		else:
-			raise VisitorSkip
-	elif node.tag == OBJECT:
+	elif node.tag == OBJECT and node.attrib['type'].startswith('image+'):
 		abs_src = node.attrib.pop('_src', None)
-		if abs_src and node.get('type').startswith('image+'):
+		if abs_src:
 			src_file = LocalFile(abs_src)
 			return _copy_image_object(notebook, new_path, src_file, node)
-		else:
-			raise VisitorSkip
-	else:
-		raise AssertionError('unknown tag')
+
+	return node
 
 
 def _replace_links_to_page_and_copy_images(notebook, old_folder, new_path, node):
@@ -164,22 +152,16 @@ def _replace_links_to_page_and_copy_images(notebook, old_folder, new_path, node)
 			my_type = link_type(abs_href)
 			if my_type == 'page':
 				target = Path(abs_href)
-				oldhref = HRef.new_from_wiki_link(node.get('href')) # *not* abs_href
+				oldhref = HRef.new_from_wiki_link(node.attrib['href']) # *not* abs_href
 				return notebook._update_link_tag(node, new_path, target, oldhref)
 			elif my_type == 'file':
 				new_href = notebook.relative_filepath(LocalFile(abs_href), new_path)
-				if new_href is None:
-					return node # could be VisitorSkip, but want to get rid of _href
-				else:
-					if node.gettext() == node.get('href'): # *not* abs_href
-						node[:] = [new_href]
-					node.set('href', new_href)
-					return node
+				if new_href is not None:
+					if node.content == [(TEXT, node.attrib['href'])]: # *not* abs_href
+						node.content[:] = [(TEXT, new_href)]
+					node.attrib['href'] = new_href
 			else:
-				logger.warn('Could not update link of type "%s": %s', my_type, abs_href)
-				raise VisitorSkip
-		else:
-			raise VisitorSkip
+				logger.warning('Could not update link of type "%s": %s', my_type, abs_href)
 	elif node.tag == IMAGE:
 		# Only copy direct attachments - else the image already was a link
 		# to a file outside of the attachment folder
@@ -190,34 +172,35 @@ def _replace_links_to_page_and_copy_images(notebook, old_folder, new_path, node)
 				return _copy_image(notebook, new_path, src_file, node)
 			else:
 				return _update_image(notebook, new_path, src_file, node)
-		else:
-			raise VisitorSkip
-	elif node.tag == OBJECT:
+	elif node.tag == OBJECT and node.attrib['type'].startswith('image+'):
 		abs_src = node.attrib.pop('_src', None)
-		if abs_src and node.get('type').startswith('image+'):
+		if abs_src:
 			src_file = LocalFile(abs_src)
 			if src_file.ischild(old_folder):
 				return _copy_image_object(notebook, new_path, src_file, node)
 			else:
 				return _update_image(notebook, new_path, src_file, node)
-		else:
-			raise VisitorSkip
 	else:
-		raise AssertionError('unknown tag')
+		pass
+
+	return node
 
 
 def _update_image(notebook, new_path, src_file, node):
 	new_src = notebook.relative_filepath(src_file, new_path)
 	if new_src is not None:
-		node.set('src', new_src)
+		node.attrib['src'] = new_src
 	return node
 
 
 def _copy_image(notebook, new_path, src_file, node):
 	folder = notebook.get_page(new_path).attachments_folder
 	new_file = folder.new_file(src_file.basename)
-	src_file.copyto(new_file)
-	node.set('src', '.' + SEP + new_file.basename)
+	if src_file.exists():
+		src_file.copyto(new_file)
+	else:
+		logger.warning('File not found: %s' % src_file.userpath)
+	node.attrib['src'] = '.' + SEP + new_file.basename
 	return node
 
 
@@ -226,5 +209,5 @@ def _copy_image_object(notebook, new_path, src_file, node):
 
 	folder = notebook.get_page(new_path).attachments_folder
 	new_file = copy_imagegenerator_src_files(src_file, folder)
-	node.set('src', '.' + SEP + new_file.basename)
+	node.attrib['src'] = '.' + SEP + new_file.basename
 	return node
